package main

import (
	"errors"
	"flow"
	"flow/registry"
	"flowserver"
	"fmt"
	"log"
	"math/rand"
	"strings"
	"time"

	"github.com/gohxs/prettylog"
)

func main() {
	prettylog.Global()

	log.Println("Running version:", flowserver.Version)

	registry.Batch{
		flow.Register("delay.cat", delayCat),
		flow.Register("delay.reverse", delayReverse),
		flow.Register("delay.split", delaySplit),
		flow.Register("delay.join", delayJoin),
		flow.Register("addduration", duration),
	}.Categories("string", "test").
		Extra("style", map[string]string{
			"color": "#a55",
		})

	registry.Batch{
		flow.Register("multiplex4", multiplex4),
		flow.Register("multiplex6", multiplex6),
		flow.Register("longduration", longduration),
		flow.Register("longduration", longduration),
		flow.Register("error", func(s string) (string, error) {
			time.Sleep(4 * time.Second)
			return "", errors.New("Some error")
		}),
	}.Categories("test").
		Extra("style", map[string]string{
			"color": "#959",
		})

	registry.Batch{
		flow.Register("str.cat", strCat),
		flow.Register("str.reverse", strReverse),
		flow.Register("str.split", strings.Split),
		flow.Register("str.join", strings.Join),
	}.Categories("string").
		Extra("style", map[string]string{
			"color": "#aa5",
		})

	addr := ":2015"
	log.Println("Starting server  at:", ":2015")

	f := flowserver.FlowServer{}
	f.ListenAndServe(addr)
}

func multiplex4(a1, a2, a3, a4 string) string {
	return strings.Join([]string{a1, a2, a3, a4}, " ")
}
func multiplex6(a1, a2, a3, a4, a5, a6 string) string {
	return strings.Join([]string{a1, a2, a3, a4, a5, a6}, " ")
}

/* Sample funcs */
func strCat(a1, a2 string) string {
	return a1 + a2
}

func strReverse(s string) string {
	n := len(s)
	runes := make([]rune, n)
	for _, r := range s {
		n--
		runes[n] = r
	}
	return string(runes[n:])
}

func delayCat(a1, a2 string) string {
	time.Sleep(time.Duration(2+rand.Intn(10)) * time.Second) // Simulate
	return strCat(a1, a2)
}

func delayReverse(s string) string {
	time.Sleep(time.Duration(2+rand.Intn(10)) * time.Second) // Simulate
	return strReverse(s)

}
func delaySplit(s, sep string) []string {
	time.Sleep(time.Duration(2+rand.Intn(10)) * time.Second) // Simulate
	return strings.Split(s, sep)
}
func delayJoin(s []string, join string) string {
	time.Sleep(time.Duration(2+rand.Intn(10)) * time.Second) // Simulate
	return strings.Join(s, join)
}

func longduration() string {
	mark := time.Now()
	time.Sleep(25 * time.Second) // Simulate
	now := time.Now()

	return " took: " + fmt.Sprint(now.Sub(mark))

}
func duration(s string) string {

	mark := time.Now()
	time.Sleep(7 * time.Second) // Simulate
	now := time.Now()

	return s + " took: " + fmt.Sprint(now.Sub(mark))
}
